#   File: allmode.R  06/03/2024
#   This R code computes the conditional mode forecast for
#   a given bandwidth. 
#     
#   The output from this program is "zn" and we can just take this
#   to the program "comb.txt" to do all the evaluations.
#
#   Reference:
#   De Gooijer, J.G. and Zerom, D. (2000).
#      Kernel based multi-step-ahead prediction of the U.S. short-term interest
#      rate. Journal of Forecasting, 19(4), 335-353.      
##############################################################################

calcul = function(y,h1=hop){  # 1)
a      = min(y)-3*h1     
b      = max(y)+3*h1     
step   = (b-a)/256     

sum1 = 0     
sum2 = 0     
i    = 1     

  while (i<=n0){     
    vect1 = (z[n, ]-x0[i,])/h1     
    vect2 = (a-y[i])/h1  
    k1    = t(vect2) %*% (vect2)
    k2    = t(vect1) %*% (vect1)  
    sum1  = sum1+( ker(k2) %*% ker(k1) )      
    sum2  = sum2+ker(k2)     
    i     = i+1     
  }
  
    if (sum2==0){     
     sum2 = 1.0e-300     
    }     

    summax = (sum1/sum2)     
    ymax   = a     
    u      = matrix(0,1,1)     
    y0     = matrix(0,1,1)     
    l      = 1     
       while (l<=256){    # 2)  
          y1   = a+l*step     
          sum1 = 0     
          sum2 = 0     
          i=1     
          while (i<=n0){  # 3)  
            vect1 = (z[n, ]-x0[i, ])/h1     
            vect2 = (y1-y[i])/h1  
            k1    = t(vect2) %*% (vect2)
	    k2    = t(vect1) %*% (vect1)  

            sum1 = sum1+(ker(k2)%*%ker(k1))     
            sum2 = sum2+ker(k2)     
            i = i+1     
          }     # 3)

       if (sum2==0){     
         sum2=1.0e-300     
       }    

       sumy = (sum1/sum2)     
       y0 = y0|y1     
       u  = u|sumy     
       

        if (sumy > summax) {    
           summax=sumy     
           ymax = y1     
        }     

        l = l+1     
        }  # 2)  

      return(ymax)     

  }    # 1) 
  
# constructing the product kernel using Gaussian distribution 

ker = function(x){(exp(-0.5*x))*((2*pi)^(-k/2))}
  

#############   MAIN    ############################

tbill9 <- read.table("C:/ali-third/Computer Code/mean-mode-median-Gauss/tbill9.txt", quote="\"", comment.char="")
data  = tbill9
datats = ts(data,freq=1)+10

n1    = nrow(data)

data1 = datats

x1 = data1[6:n1,]
x2 = data1[5:(n1-1),]
x3 = data1[4:(n1-2),]
x4 = data1[3:(n1-3),]
x5 = data1[2:(n1-4),]
x6 = data1[1:(n1-5),]

data = cbind(x1,x2,x3,x4,x5,x6)

z = data
k = 6

mx = 5
n2 = 1670
n3 = 1923
df = n3-n2+1
n  = n2

zn  = array(0,c(5,3,n3-n2+1))
zzz = matrix(0,n3-n2+1,3)

jj = 0
while (n <= n3){  # start BIG loop 1)
  jj = jj+1
  m  = 1

  znew = matrix(0,mx,3)
  while (m<=mx){ # start big loop 2)
     n0 = n-m
     y  = z[m+1:n,1]
     x0 = z[1:n0,]
     std = sd(data1[1:n])
     hop = (std)*( (n)^(-1/(k+4)))
 
     # The modal forecast at the optimal bandwidth h 
     
     ztm = calcul(y,hop)
    
     zz = cbind(n+m,data1[n+m,1],ztm) # 1671, 1672, 1673, 1674, 1675
     print(zz) 
     znew[m,] = zz

     m = m+1
 
  } # 2)
  
   zn[,,jj] = znew
   zzz[jj,] = colMeans(zn[,,jj])
   n = n+1
 } # 1)

########## OUTPUT ###################### 
print ("year, obs, conditional mode (zn) ")
zn

print ("column means (zzz) ")
zzz
     
  
  
 